@@ -22,6 +22,7 @@
 #include <svn_types.h>
 #include <svn_ra.h>
 #include <svn_path.h>
+#include <svn_props.h>
 #include <apr_file_io.h>
 #include <apr_portable.h>
 
@@ -31,7 +32,7 @@
 #include "util.h"
 #include "ra.h"
 
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 #define REPORTER_T svn_ra_reporter3_t
 #else
 #define REPORTER_T svn_ra_reporter2_t
@@ -39,11 +40,11 @@
 
 static PyObject *busy_exc;
 
-extern PyTypeObject Reporter_Type;
-extern PyTypeObject RemoteAccess_Type;
+staticforward PyTypeObject Reporter_Type;
+staticforward PyTypeObject RemoteAccess_Type;
 staticforward PyTypeObject AuthProvider_Type;
-extern PyTypeObject CredentialsIter_Type;
-extern PyTypeObject TxDeltaWindowHandler_Type;
+staticforward PyTypeObject CredentialsIter_Type;
+staticforward PyTypeObject Auth_Type;
 
 static bool ra_check_svn_path(char *path)
 {
@@ -145,8 +146,7 @@ static PyObject *reporter_set_path(PyObject *self, PyObject *args)
 						  &lock_token, &depth))
 		return NULL;
 
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
-
+#if ONLY_SINCE_SVN(1, 5)
 	RUN_SVN(reporter->reporter->set_path(reporter->report_baton, 
 												  path, revision, depth, start_empty, 
 					 lock_token, reporter->pool));
@@ -188,7 +188,7 @@ static PyObject *reporter_link_path(PyObject *self, PyObject *args)
 	if (!PyArg_ParseTuple(args, "sslb|zi", &path, &url, &revision, &start_empty, &lock_token, &depth))
 		return NULL;
 
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	RUN_SVN(reporter->reporter->link_path(reporter->report_baton, path, url, 
 				revision, depth, start_empty, lock_token, reporter->pool));
 #else
@@ -256,7 +256,7 @@ static void reporter_dealloc(PyObject *self)
 	PyObject_Del(self);
 }
 
-PyTypeObject Reporter_Type = {
+static PyTypeObject Reporter_Type = {
 	PyObject_HEAD_INIT(NULL) 0,
 	"_ra.Reporter", /*	const char *tp_name;  For printing, in format "<module>.<name>" */
 	sizeof(ReporterObject), 
@@ -343,276 +343,7 @@ static PyObject *api_version(PyObject *self)
 						 ver->patch, ver->tag);
 }
 
-static svn_error_t *py_cb_editor_set_target_revision(void *edit_baton, svn_revnum_t target_revision, apr_pool_t *pool)
-{
-	PyObject *self = (PyObject *)edit_baton, *ret;
-	PyGILState_STATE state = PyGILState_Ensure();
-
-	ret = PyObject_CallMethod(self, "set_target_revision", "l", target_revision);
-	CB_CHECK_PYRETVAL(ret);
-	Py_DECREF(ret);
-	PyGILState_Release(state);
-	return NULL;
-}
-
-static svn_error_t *py_cb_editor_open_root(void *edit_baton, svn_revnum_t base_revision, apr_pool_t *pool, void **root_baton)
-{
-	PyObject *self = (PyObject *)edit_baton, *ret;
-	PyGILState_STATE state = PyGILState_Ensure();
-	*root_baton = NULL;
-	ret = PyObject_CallMethod(self, "open_root", "l", base_revision);
-	CB_CHECK_PYRETVAL(ret);
-	*root_baton = (void *)ret;
-	PyGILState_Release(state);
-	return NULL;
-}
-
-static svn_error_t *py_cb_editor_delete_entry(const char *path, svn_revnum_t revision, void *parent_baton, apr_pool_t *pool)
-{
-	PyObject *self = (PyObject *)parent_baton, *ret;
-	PyGILState_STATE state = PyGILState_Ensure();
-	ret = PyObject_CallMethod(self, "delete_entry", "sl", path, revision);
-	CB_CHECK_PYRETVAL(ret);
-	Py_DECREF(ret);
-	PyGILState_Release(state);
-	return NULL;
-}
-
-static svn_error_t *py_cb_editor_add_directory(const char *path, void *parent_baton, const char *copyfrom_path, svn_revnum_t copyfrom_revision, apr_pool_t *pool, void **child_baton)
-{
-	PyObject *self = (PyObject *)parent_baton, *ret;
-	PyGILState_STATE state = PyGILState_Ensure();
-	*child_baton = NULL;
-
-	if (copyfrom_path == NULL) {
-		ret = PyObject_CallMethod(self, "add_directory", "s", path);
-	} else {
-		ret = PyObject_CallMethod(self, "add_directory", "ssl", path, copyfrom_path, copyfrom_revision);
-	}
-	CB_CHECK_PYRETVAL(ret);
-	*child_baton = (void *)ret;
-	PyGILState_Release(state);
-	return NULL;
-}
-
-static svn_error_t *py_cb_editor_open_directory(const char *path, void *parent_baton, svn_revnum_t base_revision, apr_pool_t *pool, void **child_baton)
-{
-	PyObject *self = (PyObject *)parent_baton, *ret;
-	PyGILState_STATE state = PyGILState_Ensure();
-	*child_baton = NULL;
-	ret = PyObject_CallMethod(self, "open_directory", "sl", path, base_revision);
-	CB_CHECK_PYRETVAL(ret);
-	*child_baton = (void *)ret;
-	PyGILState_Release(state);
-	return NULL;
-}
-
-static svn_error_t *py_cb_editor_change_prop(void *dir_baton, const char *name, const svn_string_t *value, apr_pool_t *pool)
-{
-	PyObject *self = (PyObject *)dir_baton, *ret;
-	PyGILState_STATE state = PyGILState_Ensure();
-
-	if (value != NULL) {
-		ret = PyObject_CallMethod(self, "change_prop", "sz#", name, value->data, value->len);
-	} else {
-		ret = PyObject_CallMethod(self, "change_prop", "sO", name, Py_None);
-	}
-	CB_CHECK_PYRETVAL(ret);
-	Py_DECREF(ret);
-	PyGILState_Release(state);
-	return NULL;
-}
-
-static svn_error_t *py_cb_editor_close_directory(void *dir_baton, apr_pool_t *pool)
-{
-	PyObject *self = (PyObject *)dir_baton, *ret;
-	PyGILState_STATE state = PyGILState_Ensure();
-	ret = PyObject_CallMethod(self, "close", "");
-	Py_DECREF(self);
-	CB_CHECK_PYRETVAL(ret);
-	Py_DECREF(ret);
-	PyGILState_Release(state);
-	return NULL;
-}
-
-static svn_error_t *py_cb_editor_absent_directory(const char *path, void *parent_baton, apr_pool_t *pool)
-{
-	PyObject *self = (PyObject *)parent_baton, *ret;
-	PyGILState_STATE state = PyGILState_Ensure();
-	ret = PyObject_CallMethod(self, "absent_directory", "s", path);
-	CB_CHECK_PYRETVAL(ret);
-	Py_DECREF(ret);
-	PyGILState_Release(state);
-	return NULL;
-}
-
-static svn_error_t *py_cb_editor_add_file(const char *path, void *parent_baton, const char *copy_path, svn_revnum_t copy_revision, apr_pool_t *file_pool, void **file_baton)
-{
-	PyObject *self = (PyObject *)parent_baton, *ret;
-	PyGILState_STATE state = PyGILState_Ensure();
-	if (copy_path == NULL) {
-		ret = PyObject_CallMethod(self, "add_file", "s", path);
-	} else {
-		ret = PyObject_CallMethod(self, "add_file", "ssl", path, copy_path, 
-								  copy_revision);
-	}
-	CB_CHECK_PYRETVAL(ret);
-	*file_baton = (void *)ret;
-	PyGILState_Release(state);
-	return NULL;
-}
-
-static svn_error_t *py_cb_editor_open_file(const char *path, void *parent_baton, svn_revnum_t base_revision, apr_pool_t *file_pool, void **file_baton)
-{
-	PyObject *self = (PyObject *)parent_baton, *ret;
-	PyGILState_STATE state = PyGILState_Ensure();
-	ret = PyObject_CallMethod(self, "open_file", "sl", path, base_revision);
-	CB_CHECK_PYRETVAL(ret);
-	*file_baton = (void *)ret;
-	PyGILState_Release(state);
-	return NULL;
-}
-
-static svn_error_t *py_txdelta_window_handler(svn_txdelta_window_t *window, void *baton)
-{
-	int i;
-	PyObject *ops, *ret;
-	PyObject *fn = (PyObject *)baton, *py_new_data, *py_window;
-	PyGILState_STATE state;
-	if (fn == Py_None) {
-		/* User doesn't care about deltas */
-		return NULL;
-	}
-
-	state = PyGILState_Ensure();
-
-	if (window == NULL) {
-		py_window = Py_None;
-		Py_INCREF(py_window);
-	} else {
-		ops = PyList_New(window->num_ops);
-		if (ops == NULL)
-			return NULL;
-		for (i = 0; i < window->num_ops; i++) {
-			PyObject *pyval = Py_BuildValue("(iII)", 
-											window->ops[i].action_code, 
-											window->ops[i].offset, 
-											window->ops[i].length);
-			CB_CHECK_PYRETVAL(pyval);
-			PyList_SetItem(ops, i, pyval);
-		}
-		if (window->new_data != NULL && window->new_data->data != NULL) {
-			py_new_data = PyString_FromStringAndSize(window->new_data->data, window->new_data->len);
-		} else {
-			py_new_data = Py_None;
-			Py_INCREF(py_new_data);
-		}
-		py_window = Py_BuildValue("((LIIiNN))", 
-								  window->sview_offset, 
-								  window->sview_len, 
-								  window->tview_len, 
-								  window->src_ops, ops, py_new_data);
-		CB_CHECK_PYRETVAL(py_window);
-	}
-	ret = PyObject_CallFunction(fn, "O", py_window);
-	Py_DECREF(py_window);
-	if (window == NULL) {
-		/* Signals all delta windows have been received */
-		Py_DECREF(fn);
-	}
-	CB_CHECK_PYRETVAL(ret);
-	Py_DECREF(ret);
-	PyGILState_Release(state);
-	return NULL;
-}
-
-static svn_error_t *py_cb_editor_apply_textdelta(void *file_baton, const char *base_checksum, apr_pool_t *pool, svn_txdelta_window_handler_t *handler, void **handler_baton)
-{
-	PyObject *self = (PyObject *)file_baton, *ret;
-	PyGILState_STATE state = PyGILState_Ensure();
-	*handler_baton = NULL;
-
-	ret = PyObject_CallMethod(self, "apply_textdelta", "z", base_checksum);
-	CB_CHECK_PYRETVAL(ret);
-	*handler_baton = (void *)ret;
-	*handler = py_txdelta_window_handler;
-	PyGILState_Release(state);
-	return NULL;
-}
-
-static svn_error_t *py_cb_editor_close_file(void *file_baton, 
-										 const char *text_checksum, apr_pool_t *pool)
-{
-	PyObject *self = (PyObject *)file_baton, *ret;
-	PyGILState_STATE state = PyGILState_Ensure();
-
-	if (text_checksum != NULL) {
-		ret = PyObject_CallMethod(self, "close", "");
-	} else {
-		ret = PyObject_CallMethod(self, "close", "s", text_checksum);
-	}
-	Py_DECREF(self);
-	CB_CHECK_PYRETVAL(ret);
-	Py_DECREF(ret);
-	PyGILState_Release(state);
-	return NULL;
-}
-
-static svn_error_t *py_cb_editor_absent_file(const char *path, void *parent_baton, apr_pool_t *pool)
-{
-	PyObject *self = (PyObject *)parent_baton, *ret;
-	PyGILState_STATE state = PyGILState_Ensure();
-	ret = PyObject_CallMethod(self, "absent_file", "s", path);
-	CB_CHECK_PYRETVAL(ret);
-	Py_DECREF(ret);
-	PyGILState_Release(state);
-	return NULL;
-}
-
-static svn_error_t *py_cb_editor_close_edit(void *edit_baton, apr_pool_t *pool)
-{
-	PyObject *self = (PyObject *)edit_baton, *ret;
-	PyGILState_STATE state = PyGILState_Ensure();
-	ret = PyObject_CallMethod(self, "close", "");
-	Py_DECREF(self);
-	CB_CHECK_PYRETVAL(ret);
-	Py_DECREF(ret);
-	PyGILState_Release(state);
-	return NULL;
-}
-
-static svn_error_t *py_cb_editor_abort_edit(void *edit_baton, apr_pool_t *pool)
-{
-	PyObject *self = (PyObject *)edit_baton, *ret;
-	PyGILState_STATE state = PyGILState_Ensure();
-	ret = PyObject_CallMethod(self, "abort", "");
-	Py_DECREF(self);
-	CB_CHECK_PYRETVAL(ret);
-	Py_DECREF(ret);
-	PyGILState_Release(state);
-	return NULL;
-}
-
-static const svn_delta_editor_t py_editor = {
-	py_cb_editor_set_target_revision,
-	py_cb_editor_open_root,
-	py_cb_editor_delete_entry,
-	py_cb_editor_add_directory,
-	py_cb_editor_open_directory,
-	py_cb_editor_change_prop,
-	py_cb_editor_close_directory,
-	py_cb_editor_absent_directory,
-	py_cb_editor_add_file,
-	py_cb_editor_open_file,
-	py_cb_editor_apply_textdelta,
-	py_cb_editor_change_prop,
-	py_cb_editor_close_file,
-	py_cb_editor_absent_file,
-	py_cb_editor_close_edit,
-	py_cb_editor_abort_edit
-};
-
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 static svn_error_t *py_file_rev_handler(void *baton, const char *path, svn_revnum_t rev, apr_hash_t *rev_props, svn_boolean_t result_of_merge, svn_txdelta_window_handler_t *delta_handler, void **delta_baton, apr_array_header_t *prop_diffs, apr_pool_t *pool)
 {
 	PyObject *fn = (PyObject *)baton, *ret, *py_rev_props;
@@ -691,7 +422,7 @@ static bool ra_check_busy(RemoteAccessObject *raobj)
 	return false;
 }
 
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 static svn_error_t *py_get_client_string(void *baton, const char **name, apr_pool_t *pool)
 {
 	RemoteAccessObject *self = (RemoteAccessObject *)baton;
@@ -731,7 +462,7 @@ static svn_error_t *py_open_tmp_file(apr_file_t **fp, void *callback,
 
 		SVN_ERR (svn_io_temp_dir (&path, pool));
 		path = svn_path_join (path, "subvertpy", pool);
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 6
+#if ONLY_SINCE_SVN(1, 6)
 		SVN_ERR (svn_io_open_unique_file3(fp, NULL, path, svn_io_file_del_on_pool_cleanup, pool, pool));
 #else
 		SVN_ERR (svn_io_open_unique_file (fp, NULL, path, ".tmp", TRUE, pool));
@@ -854,7 +585,7 @@ static PyObject *ra_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 	Py_INCREF(progress_cb);
 	ret->progress_func = progress_cb;
 	callbacks2->progress_baton = (void *)ret;
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	callbacks2->get_client_string = py_get_client_string;
 #endif
 	config_hash = config_hash_from_object(config, ret->pool);
@@ -863,7 +594,7 @@ static PyObject *ra_new(PyTypeObject *type, PyObject *args, PyObject *kwargs)
 		return NULL;
 	}
 	Py_BEGIN_ALLOW_THREADS
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	err = svn_ra_open3(&ret->ra, ret->url, uuid,
 			   callbacks2, ret, config_hash, ret->pool);
 #else
@@ -901,7 +632,7 @@ static PyObject *ra_get_uuid(PyObject *self)
 	temp_pool = Pool(NULL);
 	if (temp_pool == NULL)
 		return NULL;
-#if SVN_VER_MAJOR == 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	RUN_RA_WITH_POOL(temp_pool, ra, svn_ra_get_uuid2(ra->ra, &uuid, temp_pool));
 #else
 	RUN_RA_WITH_POOL(temp_pool, ra, svn_ra_get_uuid(ra->ra, &uuid, temp_pool));
@@ -990,7 +721,7 @@ static PyObject *ra_get_log(PyObject *self, PyObject *args, PyObject *kwargs)
 		return NULL;
 	}
 
-#if SVN_VER_MAJOR <= 1 && SVN_VER_MINOR < 5
+#if ONLY_BEFORE_SVN(1, 5)
 	if (revprops == Py_None) {
 		PyErr_SetString(PyExc_NotImplementedError,
 			"fetching all revision properties not supported");
@@ -1028,7 +759,7 @@ static PyObject *ra_get_log(PyObject *self, PyObject *args, PyObject *kwargs)
 		return NULL;
 	}
 
-#if SVN_VER_MAJOR == 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	RUN_RA_WITH_POOL(temp_pool, ra, svn_ra_get_log2(ra->ra, 
 			apr_paths, start, end, limit,
 			discover_changed_paths, strict_node_history, 
@@ -1062,7 +793,7 @@ static PyObject *ra_get_repos_root(PyObject *self)
 		temp_pool = Pool(NULL);
 		if (temp_pool == NULL)
 			return NULL;
-#if SVN_VER_MAJOR == 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 		RUN_RA_WITH_POOL(temp_pool, ra,
 						  svn_ra_get_repos_root2(ra->ra, &root, temp_pool));
 #else
@@ -1089,7 +820,7 @@ static PyObject *ra_get_url(PyObject *self, void *closure)
 	if (ra_check_busy(ra))
 		return NULL;
 
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	temp_pool = Pool(NULL);
 
 	RUN_RA_WITH_POOL(temp_pool, ra,
@@ -1133,7 +864,7 @@ static PyObject *ra_do_update(PyObject *self, PyObject *args)
 		return NULL;
 
 	Py_INCREF(update_editor);
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	Py_BEGIN_ALLOW_THREADS
 	err = svn_ra_do_update2(ra->ra, &reporter, 
 												  &report_baton, 
@@ -1202,7 +933,7 @@ static PyObject *ra_do_switch(PyObject *self, PyObject *args)
 	Py_INCREF(update_editor);
 	Py_BEGIN_ALLOW_THREADS
 
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	err = svn_ra_do_switch2(
 						ra->ra, &reporter, &report_baton, 
 						revision_to_update_to, update_target, 
@@ -1259,7 +990,7 @@ static PyObject *ra_do_diff(PyObject *self, PyObject *args)
 
 	Py_INCREF(diff_editor);
 	Py_BEGIN_ALLOW_THREADS
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	err = svn_ra_do_diff3(ra->ra, &reporter, &report_baton, 
 												  revision_to_update_to, 
 												  diff_target, recurse?svn_depth_infinity:svn_depth_files,
@@ -1327,7 +1058,7 @@ static PyObject *ra_replay(PyObject *self, PyObject *args)
 	Py_RETURN_NONE;
 }
 
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 static svn_error_t *py_revstart_cb(svn_revnum_t revision, void *replay_baton,
    const svn_delta_editor_t **editor, void **edit_baton, apr_hash_t *rev_props, apr_pool_t *pool)
 {
@@ -1370,7 +1101,7 @@ static svn_error_t *py_revfinish_cb(svn_revnum_t revision, void *replay_baton,
 
 static PyObject *ra_replay_range(PyObject *self, PyObject *args)
 {
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	RemoteAccessObject *ra = (RemoteAccessObject *)self;
 	apr_pool_t *temp_pool;
 	svn_revnum_t start_revision, end_revision, low_water_mark;
@@ -1443,10 +1174,10 @@ static PyObject *get_commit_editor(PyObject *self, PyObject *args, PyObject *kwa
 	void *edit_baton;
 	RemoteAccessObject *ra = (RemoteAccessObject *)self;
 	apr_hash_t *hash_lock_tokens;
-#if SVN_VER_MAJOR == 1 && SVN_VER_MINOR < 5
-	PyObject *py_log_msg;
-#else
+#if ONLY_SINCE_SVN(1, 5)
 	apr_hash_t *hash_revprops;
+#else
+	PyObject *py_log_msg;
 #endif
 	svn_error_t *err;
 
@@ -1477,7 +1208,7 @@ static PyObject *get_commit_editor(PyObject *self, PyObject *args, PyObject *kwa
 	if (ra_check_busy(ra))
 		return NULL;
 
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	hash_revprops = prop_dict_to_hash(pool, revprops);
 	if (hash_revprops == NULL) {
 		apr_pool_destroy(pool);
@@ -1559,7 +1290,7 @@ static PyObject *ra_change_rev_prop(PyObject *self, PyObject *args)
 }
 
 
-static PyObject *ra_get_dir(PyObject *self, PyObject *args)
+static PyObject *ra_get_dir(PyObject *self, PyObject *args, PyObject *kwargs)
 {
 	apr_pool_t *temp_pool;
 	apr_hash_t *dirents;
@@ -1572,10 +1303,12 @@ static PyObject *ra_get_dir(PyObject *self, PyObject *args)
 	apr_ssize_t klen;
 	char *path;
 	svn_revnum_t revision = -1;
-	long dirent_fields = 0;
+	unsigned int dirent_fields = 0;
 	PyObject *py_dirents, *py_props;
+    char *kwnames[] = { "path", "revision", "fields", NULL };
 
-	if (!PyArg_ParseTuple(args, "s|lI", &path, &revision, &dirent_fields))
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s|lI", kwnames,
+                                     &path, &revision, &dirent_fields))
 		return NULL;
 
 	if (ra_check_busy(ra))
@@ -1742,7 +1475,7 @@ static PyObject *ra_stat(PyObject *self, PyObject *args)
 
 static PyObject *ra_has_capability(PyObject *self, PyObject *args)
 {
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	char *capability;
 	apr_pool_t *temp_pool;
 	RemoteAccessObject *ra = (RemoteAccessObject *)self;
@@ -1928,7 +1661,7 @@ static PyObject *ra_get_locations(PyObject *self, PyObject *args)
 	return ret;
 }
 
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 static PyObject *range_to_tuple(svn_merge_range_t *range)
 {
 	return Py_BuildValue("(llb)", range->start, range->end, range->inheritable);
@@ -1975,7 +1708,7 @@ static PyObject *mergeinfo_to_dict(svn_mergeinfo_t mergeinfo, apr_pool_t *temp_p
 
 static PyObject *ra_mergeinfo(PyObject *self, PyObject *args)
 {
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	RemoteAccessObject *ra = (RemoteAccessObject *)self;
 	apr_array_header_t *apr_paths;
 	apr_pool_t *temp_pool;
@@ -2033,7 +1766,7 @@ static PyObject *ra_mergeinfo(PyObject *self, PyObject *args)
 #endif
 }
 
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 static svn_error_t *py_location_segment_receiver(svn_location_segment_t *segment, void *baton, apr_pool_t *pool)
 {
 	PyObject *fn = baton, *ret;
@@ -2049,7 +1782,7 @@ static svn_error_t *py_location_segment_receiver(svn_location_segment_t *segment
 
 static PyObject *ra_get_location_segments(PyObject *self, PyObject *args)
 {
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	RemoteAccessObject *ra = (RemoteAccessObject *)self;
 	svn_revnum_t peg_revision, start_revision, end_revision;
 	char *path;
@@ -2104,7 +1837,7 @@ static PyObject *ra_get_file_revs(PyObject *self, PyObject *args)
 	if (temp_pool == NULL)
 		return NULL;
 
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	RUN_RA_WITH_POOL(temp_pool, ra, svn_ra_get_file_revs2(ra->ra, path, start, end, 
 				include_merged_revisions, 
 				py_file_rev_handler, (void *)file_rev_handler, 
@@ -2185,7 +1918,7 @@ static PyMethodDef ra_methods[] = {
 	{ "get_lock", ra_get_lock, METH_VARARGS, 
 		"S.get_lock(path) -> lock\n"
 	},
-	{ "get_dir", ra_get_dir, METH_VARARGS, 
+	{ "get_dir", ra_get_dir, METH_VARARGS|METH_KEYWORDS, 
 		"S.get_dir(path, revision, dirent_fields=-1) -> (dirents, fetched_rev, properties)\n"
 		"Get the contents of a directory. "},
 	{ "get_file", ra_get_file, METH_VARARGS, 
@@ -2207,8 +1940,8 @@ static PyMethodDef ra_methods[] = {
 		"S.replay_range(start_rev, end_rev, low_water_mark, cbs, send_deltas=True)\n"
 		"Replay a range of revisions, reporting them to an update editor.\n"
 		"cbs is a two-tuple with two callbacks:\n"
-		"  start_rev_cb(revision, revprops) -> editor\n"
-		"  finish_rev_cb(revision, revprops, editor)\n"
+		"- start_rev_cb(revision, revprops) -> editor\n"
+		"- finish_rev_cb(revision, revprops, editor)\n"
 	},
 	{ "do_switch", ra_do_switch, METH_VARARGS, 
 		"S.do_switch(revision_to_update_to, update_target, recurse, switch_url, update_editor)\n" },
@@ -2251,7 +1984,7 @@ static PyMemberDef ra_members[] = {
 	{ NULL, }
 };
 
-PyTypeObject RemoteAccess_Type = {
+static PyTypeObject RemoteAccess_Type = {
 	PyObject_HEAD_INIT(NULL) 0,
 	"_ra.RemoteAccess", /*	const char *tp_name;  For printing, in format "<module>.<name>" */
 	sizeof(RemoteAccessObject), 
@@ -2535,7 +2268,7 @@ static PyObject *credentials_iter_next(CredentialsIterObject *iterator)
 	return ret;
 }
 
-PyTypeObject CredentialsIter_Type = {
+static PyTypeObject CredentialsIter_Type = {
 	PyObject_HEAD_INIT(NULL) 0,
 	"_ra.CredentialsIter", /*	const char *tp_name;  For printing, in format "<module>.<name>" */
 	sizeof(CredentialsIterObject),
@@ -2612,7 +2345,7 @@ static void auth_dealloc(PyObject *self)
 	Py_XDECREF(auth->providers);
 }
 
-PyTypeObject Auth_Type = {
+static PyTypeObject Auth_Type = {
 	PyObject_HEAD_INIT(NULL) 0,
 	"_ra.Auth", /*	const char *tp_name;  For printing, in format "<module>.<name>" */
 	sizeof(AuthObject),
@@ -3031,7 +2764,7 @@ static PyObject *get_username_provider(PyObject *self)
 	return (PyObject *)auth;
 }
 
-#if SVN_VER_MAJOR == 1 && SVN_VER_MINOR >= 6
+#if ONLY_SINCE_SVN(1, 6)
 static svn_error_t *py_cb_get_simple_provider_prompt(svn_boolean_t *may_save_plaintext,
                                                      const char *realmstring,
                                                      void *baton,
@@ -3071,7 +2804,7 @@ static PyObject *get_simple_provider(PyObject *self, PyObject *args)
 	auth->pool = Pool(NULL);
 	if (auth->pool == NULL)
 		return NULL;
-#if SVN_VER_MAJOR == 1 && SVN_VER_MINOR >= 6
+#if ONLY_SINCE_SVN(1, 6)
 	Py_INCREF(callback);
 	auth->callback = callback;
 	svn_auth_get_simple_provider2(&auth->provider, py_cb_get_simple_provider_prompt, callback, auth->pool);
@@ -3122,7 +2855,7 @@ static PyObject *get_ssl_client_cert_pw_file_provider(PyObject *self)
 	if (auth->pool == NULL)
 		return NULL;
 
-#if SVN_VER_MAJOR == 1 && SVN_VER_MINOR >= 6
+#if ONLY_SINCE_SVN(1, 6)
 	svn_auth_get_ssl_client_cert_pw_file_provider2(&auth->provider, NULL, NULL, auth->pool);
 #else
 	svn_auth_get_ssl_client_cert_pw_file_provider(&auth->provider, auth->pool);
@@ -3168,7 +2901,7 @@ static PyObject *get_windows_simple_provider(PyObject* self)
 	return (PyObject *)auth;
 }
 
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 static PyObject *get_windows_ssl_server_trust_provider(PyObject *self)
 {
 	AuthProviderObject *auth = PyObject_New(AuthProviderObject, &AuthProvider_Type);
@@ -3201,7 +2934,7 @@ static PyObject *get_keychain_simple_provider(PyObject* self)
 
 static PyObject *get_platform_specific_client_providers(PyObject *self)
 {
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 6
+#if ONLY_SINCE_SVN(1, 6)
 	/* svn_auth_get_platform_specific_client_providers() allocates all the
 	 * providers in a single pool, so we can't use it :/ */
 	const char *provider_names[] = {
@@ -3257,7 +2990,7 @@ static PyObject *get_platform_specific_client_providers(PyObject *self)
 	PyList_Append(pylist, provider);
 	Py_DECREF(provider);
 
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	provider = get_windows_ssl_server_trust_provider(self);
 	if (provider == NULL)
 		return NULL;
@@ -3292,7 +3025,7 @@ static PyMethodDef ra_module_methods[] = {
 	{ "get_simple_provider", (PyCFunction)get_simple_provider, METH_VARARGS, NULL },
 #if defined(WIN32)
 	{ "get_windows_simple_provider", (PyCFunction)get_windows_simple_provider, METH_NOARGS, NULL },
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	{ "get_windows_ssl_server_trust_provider", (PyCFunction)get_windows_ssl_server_trust_provider, METH_NOARGS, NULL },
 #endif
 #endif
@@ -3382,7 +3115,7 @@ void init_ra(void)
 	PyModule_AddIntConstant(mod, "DIRENT_LAST_AUTHOR", SVN_DIRENT_LAST_AUTHOR);
 	PyModule_AddIntConstant(mod, "DIRENT_ALL", SVN_DIRENT_ALL);
 
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 	PyModule_AddIntConstant(mod, "MERGEINFO_EXPLICIT", svn_mergeinfo_explicit);
 	PyModule_AddIntConstant(mod, "MERGEINFO_INHERITED", svn_mergeinfo_inherited);
 	PyModule_AddIntConstant(mod, "MERGEINFO_NEAREST_ANCESTOR", svn_mergeinfo_nearest_ancestor);
