@@ -740,3 +740,274 @@ void initeditor(void)
     FileEditor_Type.tp_dealloc = (destructor)PyObject_Del;
     DirectoryEditor_Type.tp_dealloc = (destructor)PyObject_Del;
 }
+
+static svn_error_t *py_cb_editor_set_target_revision(void *edit_baton, svn_revnum_t target_revision, apr_pool_t *pool)
+{
+	PyObject *self = (PyObject *)edit_baton, *ret;
+	PyGILState_STATE state = PyGILState_Ensure();
+
+	ret = PyObject_CallMethod(self, "set_target_revision", "l", target_revision);
+	CB_CHECK_PYRETVAL(ret);
+	Py_DECREF(ret);
+	PyGILState_Release(state);
+	return NULL;
+}
+
+static svn_error_t *py_cb_editor_open_root(void *edit_baton, svn_revnum_t base_revision, apr_pool_t *pool, void **root_baton)
+{
+	PyObject *self = (PyObject *)edit_baton, *ret;
+	PyGILState_STATE state = PyGILState_Ensure();
+	*root_baton = NULL;
+	ret = PyObject_CallMethod(self, "open_root", "l", base_revision);
+	CB_CHECK_PYRETVAL(ret);
+	*root_baton = (void *)ret;
+	PyGILState_Release(state);
+	return NULL;
+}
+
+static svn_error_t *py_cb_editor_delete_entry(const char *path, svn_revnum_t revision, void *parent_baton, apr_pool_t *pool)
+{
+	PyObject *self = (PyObject *)parent_baton, *ret;
+	PyGILState_STATE state = PyGILState_Ensure();
+	ret = PyObject_CallMethod(self, "delete_entry", "sl", path, revision);
+	CB_CHECK_PYRETVAL(ret);
+	Py_DECREF(ret);
+	PyGILState_Release(state);
+	return NULL;
+}
+
+static svn_error_t *py_cb_editor_add_directory(const char *path, void *parent_baton, const char *copyfrom_path, svn_revnum_t copyfrom_revision, apr_pool_t *pool, void **child_baton)
+{
+	PyObject *self = (PyObject *)parent_baton, *ret;
+	PyGILState_STATE state = PyGILState_Ensure();
+	*child_baton = NULL;
+
+	if (copyfrom_path == NULL) {
+		ret = PyObject_CallMethod(self, "add_directory", "s", path);
+	} else {
+		ret = PyObject_CallMethod(self, "add_directory", "ssl", path, copyfrom_path, copyfrom_revision);
+	}
+	CB_CHECK_PYRETVAL(ret);
+	*child_baton = (void *)ret;
+	PyGILState_Release(state);
+	return NULL;
+}
+
+static svn_error_t *py_cb_editor_open_directory(const char *path, void *parent_baton, svn_revnum_t base_revision, apr_pool_t *pool, void **child_baton)
+{
+	PyObject *self = (PyObject *)parent_baton, *ret;
+	PyGILState_STATE state = PyGILState_Ensure();
+	*child_baton = NULL;
+	ret = PyObject_CallMethod(self, "open_directory", "sl", path, base_revision);
+	CB_CHECK_PYRETVAL(ret);
+	*child_baton = (void *)ret;
+	PyGILState_Release(state);
+	return NULL;
+}
+
+static svn_error_t *py_cb_editor_change_prop(void *dir_baton, const char *name, const svn_string_t *value, apr_pool_t *pool)
+{
+	PyObject *self = (PyObject *)dir_baton, *ret;
+	PyGILState_STATE state = PyGILState_Ensure();
+
+	if (value != NULL) {
+		ret = PyObject_CallMethod(self, "change_prop", "sz#", name, value->data, value->len);
+	} else {
+		ret = PyObject_CallMethod(self, "change_prop", "sO", name, Py_None);
+	}
+	CB_CHECK_PYRETVAL(ret);
+	Py_DECREF(ret);
+	PyGILState_Release(state);
+	return NULL;
+}
+
+static svn_error_t *py_cb_editor_close_directory(void *dir_baton, apr_pool_t *pool)
+{
+	PyObject *self = (PyObject *)dir_baton, *ret;
+	PyGILState_STATE state = PyGILState_Ensure();
+	ret = PyObject_CallMethod(self, "close", "");
+	Py_DECREF(self);
+	CB_CHECK_PYRETVAL(ret);
+	Py_DECREF(ret);
+	PyGILState_Release(state);
+	return NULL;
+}
+
+static svn_error_t *py_cb_editor_absent_directory(const char *path, void *parent_baton, apr_pool_t *pool)
+{
+	PyObject *self = (PyObject *)parent_baton, *ret;
+	PyGILState_STATE state = PyGILState_Ensure();
+	ret = PyObject_CallMethod(self, "absent_directory", "s", path);
+	CB_CHECK_PYRETVAL(ret);
+	Py_DECREF(ret);
+	PyGILState_Release(state);
+	return NULL;
+}
+
+static svn_error_t *py_cb_editor_add_file(const char *path, void *parent_baton, const char *copy_path, svn_revnum_t copy_revision, apr_pool_t *file_pool, void **file_baton)
+{
+	PyObject *self = (PyObject *)parent_baton, *ret;
+	PyGILState_STATE state = PyGILState_Ensure();
+	if (copy_path == NULL) {
+		ret = PyObject_CallMethod(self, "add_file", "s", path);
+	} else {
+		ret = PyObject_CallMethod(self, "add_file", "ssl", path, copy_path, 
+								  copy_revision);
+	}
+	CB_CHECK_PYRETVAL(ret);
+	*file_baton = (void *)ret;
+	PyGILState_Release(state);
+	return NULL;
+}
+
+static svn_error_t *py_cb_editor_open_file(const char *path, void *parent_baton, svn_revnum_t base_revision, apr_pool_t *file_pool, void **file_baton)
+{
+	PyObject *self = (PyObject *)parent_baton, *ret;
+	PyGILState_STATE state = PyGILState_Ensure();
+	ret = PyObject_CallMethod(self, "open_file", "sl", path, base_revision);
+	CB_CHECK_PYRETVAL(ret);
+	*file_baton = (void *)ret;
+	PyGILState_Release(state);
+	return NULL;
+}
+
+svn_error_t *py_txdelta_window_handler(svn_txdelta_window_t *window, void *baton)
+{
+	int i;
+	PyObject *ops, *ret;
+	PyObject *fn = (PyObject *)baton, *py_new_data, *py_window;
+	PyGILState_STATE state;
+	if (fn == Py_None) {
+		/* User doesn't care about deltas */
+		return NULL;
+	}
+
+	state = PyGILState_Ensure();
+
+	if (window == NULL) {
+		py_window = Py_None;
+		Py_INCREF(py_window);
+	} else {
+		ops = PyList_New(window->num_ops);
+		if (ops == NULL)
+			return NULL;
+		for (i = 0; i < window->num_ops; i++) {
+			PyObject *pyval = Py_BuildValue("(iII)", 
+											window->ops[i].action_code, 
+											window->ops[i].offset, 
+											window->ops[i].length);
+			CB_CHECK_PYRETVAL(pyval);
+			PyList_SetItem(ops, i, pyval);
+		}
+		if (window->new_data != NULL && window->new_data->data != NULL) {
+			py_new_data = PyString_FromStringAndSize(window->new_data->data, window->new_data->len);
+		} else {
+			py_new_data = Py_None;
+			Py_INCREF(py_new_data);
+		}
+		py_window = Py_BuildValue("((LIIiNN))", 
+								  window->sview_offset, 
+								  window->sview_len, 
+								  window->tview_len, 
+								  window->src_ops, ops, py_new_data);
+		CB_CHECK_PYRETVAL(py_window);
+	}
+	ret = PyObject_CallFunction(fn, "O", py_window);
+	Py_DECREF(py_window);
+	if (window == NULL) {
+		/* Signals all delta windows have been received */
+		Py_DECREF(fn);
+	}
+	CB_CHECK_PYRETVAL(ret);
+	Py_DECREF(ret);
+	PyGILState_Release(state);
+	return NULL;
+}
+
+static svn_error_t *py_cb_editor_apply_textdelta(void *file_baton, const char *base_checksum, apr_pool_t *pool, svn_txdelta_window_handler_t *handler, void **handler_baton)
+{
+	PyObject *self = (PyObject *)file_baton, *ret;
+	PyGILState_STATE state = PyGILState_Ensure();
+	*handler_baton = NULL;
+
+	ret = PyObject_CallMethod(self, "apply_textdelta", "z", base_checksum);
+	CB_CHECK_PYRETVAL(ret);
+	*handler_baton = (void *)ret;
+	*handler = py_txdelta_window_handler;
+	PyGILState_Release(state);
+	return NULL;
+}
+
+static svn_error_t *py_cb_editor_close_file(void *file_baton, 
+										 const char *text_checksum, apr_pool_t *pool)
+{
+	PyObject *self = (PyObject *)file_baton, *ret;
+	PyGILState_STATE state = PyGILState_Ensure();
+
+	if (text_checksum != NULL) {
+		ret = PyObject_CallMethod(self, "close", "");
+	} else {
+		ret = PyObject_CallMethod(self, "close", "s", text_checksum);
+	}
+	Py_DECREF(self);
+	CB_CHECK_PYRETVAL(ret);
+	Py_DECREF(ret);
+	PyGILState_Release(state);
+	return NULL;
+}
+
+static svn_error_t *py_cb_editor_absent_file(const char *path, void *parent_baton, apr_pool_t *pool)
+{
+	PyObject *self = (PyObject *)parent_baton, *ret;
+	PyGILState_STATE state = PyGILState_Ensure();
+	ret = PyObject_CallMethod(self, "absent_file", "s", path);
+	CB_CHECK_PYRETVAL(ret);
+	Py_DECREF(ret);
+	PyGILState_Release(state);
+	return NULL;
+}
+
+static svn_error_t *py_cb_editor_close_edit(void *edit_baton, apr_pool_t *pool)
+{
+	PyObject *self = (PyObject *)edit_baton, *ret;
+	PyGILState_STATE state = PyGILState_Ensure();
+	ret = PyObject_CallMethod(self, "close", "");
+	Py_DECREF(self);
+	CB_CHECK_PYRETVAL(ret);
+	Py_DECREF(ret);
+	PyGILState_Release(state);
+	return NULL;
+}
+
+static svn_error_t *py_cb_editor_abort_edit(void *edit_baton, apr_pool_t *pool)
+{
+	PyObject *self = (PyObject *)edit_baton, *ret;
+	PyGILState_STATE state = PyGILState_Ensure();
+	ret = PyObject_CallMethod(self, "abort", "");
+	Py_DECREF(self);
+	CB_CHECK_PYRETVAL(ret);
+	Py_DECREF(ret);
+	PyGILState_Release(state);
+	return NULL;
+}
+
+const svn_delta_editor_t py_editor = {
+	py_cb_editor_set_target_revision,
+	py_cb_editor_open_root,
+	py_cb_editor_delete_entry,
+	py_cb_editor_add_directory,
+	py_cb_editor_open_directory,
+	py_cb_editor_change_prop,
+	py_cb_editor_close_directory,
+	py_cb_editor_absent_directory,
+	py_cb_editor_add_file,
+	py_cb_editor_open_file,
+	py_cb_editor_apply_textdelta,
+	py_cb_editor_change_prop,
+	py_cb_editor_close_file,
+	py_cb_editor_absent_file,
+	py_cb_editor_close_edit,
+	py_cb_editor_abort_edit
+};
+
+
