@@ -43,4 +43,8 @@ typedef struct {
 
 extern void initeditor(void);
 
+svn_error_t *py_txdelta_window_handler(svn_txdelta_window_t *window, void *baton);
+
+const svn_delta_editor_t py_editor;
+
 #endif /* _BZR_SVN_EDITOR_H_ */
