@@ -43,7 +43,6 @@ void PyErr_SetAprStatus(apr_status_t status)
 		apr_strerror(status, errmsg, sizeof(errmsg)));
 }
 
-
 apr_pool_t *Pool(apr_pool_t *parent)
 {
 	apr_status_t status;
@@ -77,6 +76,26 @@ PyTypeObject *PyErr_GetSubversionExceptionTypeObject(void)
 	return (PyTypeObject *)excobj;
 }
 
+PyTypeObject *PyErr_GetGaiExceptionTypeObject(void)
+{
+	PyObject *socketmod, *excobj;
+	socketmod = PyImport_ImportModule("socket");
+
+	if (socketmod == NULL) {
+		return NULL;
+	}
+
+	excobj = PyObject_GetAttrString(socketmod, "gaierror");
+	Py_DECREF(socketmod);
+
+	if (excobj == NULL) {
+		PyErr_BadInternalCall();
+		return NULL;
+	}
+
+	return (PyTypeObject *)excobj;
+}
+
 PyObject *PyErr_NewSubversionException(svn_error_t *error)
 {
 	PyObject *loc, *child;
@@ -103,7 +122,7 @@ PyObject *PyErr_NewSubversionException(svn_error_t *error)
 		Py_INCREF(child);
 	}
 
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 4
+#if ONLY_SINCE_SVN(1, 4)
 	message = svn_err_best_message(error, buf, sizeof(buf));
 #else
 	message = error->message;
@@ -124,13 +143,30 @@ void PyErr_SetSubversionException(svn_error_t *error)
 	}
 
 	if (error->apr_err >= APR_OS_START_SYSERR && 
-		error->apr_err < APR_OS_START_SYSERR + 1000) {
+		error->apr_err < APR_OS_START_SYSERR + APR_OS_ERRSPACE_SIZE) {
 		PyObject *excval = Py_BuildValue("(iz)", error->apr_err - APR_OS_START_SYSERR, error->message);
 		PyErr_SetObject(PyExc_OSError, excval);
 		Py_DECREF(excval);
 		return;
 	}
 
+	if (error->apr_err >= APR_OS_START_EAIERR &&
+		error->apr_err < APR_OS_START_EAIERR + APR_OS_ERRSPACE_SIZE) {
+		excobj = (PyObject *)PyErr_GetGaiExceptionTypeObject();
+		if (excobj == NULL)
+			return;
+
+		excval = Py_BuildValue("(is)", error->apr_err - APR_OS_START_EAIERR,
+							   error->message);
+		if (excval == NULL)
+			return;
+
+		PyErr_SetObject(excobj, excval);
+		Py_DECREF(excval);
+		Py_DECREF(excobj);
+		return;
+	}
+
 	excobj = (PyObject *)PyErr_GetSubversionExceptionTypeObject();
 	if (excobj == NULL)
 		return;
@@ -340,7 +376,7 @@ PyObject *pyify_changed_paths(apr_hash_t *changed_paths, bool node_kind, apr_poo
 	return py_changed_paths;
 }
 
-#if SVN_VER_MAJOR == 1 && SVN_VER_MINOR >= 6
+#if ONLY_SINCE_SVN(1, 6)
 PyObject *pyify_changed_paths2(apr_hash_t *changed_paths, apr_pool_t *pool)
 {
 	PyObject *py_changed_paths, *pyval;
@@ -370,7 +406,7 @@ PyObject *pyify_changed_paths2(apr_hash_t *changed_paths, apr_pool_t *pool)
 }
 #endif
 
-#if SVN_VER_MAJOR == 1 && SVN_VER_MINOR >= 5
+#if ONLY_SINCE_SVN(1, 5)
 svn_error_t *py_svn_log_entry_receiver(void *baton, svn_log_entry_t *log_entry, apr_pool_t *pool)
 {
 	PyObject *revprops, *py_changed_paths, *ret;
@@ -738,7 +774,7 @@ static PyObject *stream_read(StreamObject *self, PyObject *args)
 		apr_pool_destroy(temp_pool);
 		return ret;
 	} else {
-#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 6
+#if ONLY_SINCE_SVN(1, 6)
 		svn_string_t *result;
 		RUN_SVN_WITH_POOL(temp_pool, svn_string_from_stream(&result, 
 							   self->stream,
@@ -798,7 +834,7 @@ PyTypeObject Stream_Type = {
 	/* Flags to define presence of optional/expanded features */
 	0, /*	long tp_flags;	*/
 	
-	NULL, /*	const char *tp_doc;  Documentation string */
+	"Byte stream", /*	const char *tp_doc;  Documentation string */
 	
 	/* Assigned meaning in release 2.0 */
 	/* call function for all accessible objects */
